angular.module('starter.services', ['ngCordova'])

.factory('$localstorage', ['$window', function($window) {
  return {
    set: function(key, value) {
      $window.localStorage[key] = value;
    },
    get: function(key, defaultValue) {
      if (!$window.localStorage[key]) {
        $window.localStorage[key] = defaultValue;
      };
      return $window.localStorage[key];
    },
    setObject: function(key, value) {
      $window.localStorage[key] = JSON.stringify(value);
    },
    getObject: function(key) {
      return JSON.parse($window.localStorage[key] || '{}');
    }
  }
}])

.factory('$filechooser', ['$q', '$window', function($q, $window) {
  return {
    get: function() {
      var q = $q.defer();
      
      if ($window.plugins && $window.plugins.mfilechooser) {
        $window.plugins.mfilechooser.open([], function(filePath) {
          q.resolve(filePath);
        }, function(error) {
          q.reject(error);
        });
      } else {
        q.reject();
      }
      return q.promise
    }
  }
}])

.factory('Archive', ['$q', '$cordovaFile', '$localstorage', function($q, $cordovaFile, $localstorage) {

  var biliPath = $localstorage.get("biliPath");

  //e.g file:///storage/emulated/0/Android/data/tv.danmaku.bili/download/1606322/ 
  var formatURI = function(url) {
    return url.substring(7)
  }; 

  var addAVPageDir = function(uri, page) {
    var uri = uri + page;
    return $cordovaFile.createDir(uri, false)
  }

  var addAVDir = function(avid) {
    var uri = biliPath + "download/" + avid;
    return $cordovaFile.createDir(uri, false);
  }
  
  var writeIndexJson = function(filePath) {
      var json = {
            "type_tag":"lua.flv.bapi.2",
            "available_period_milli":-1,
            "from":"vupload",
            "parsed_milli":-1,
            "need_faad":false,
            "local_proxy_type":0,
            "index_mrl":"http\/vsl:\/\/index_mrl",
            "is_resolved":true,
            "prefer_vlc":false,
            "player_codec_config_list": [
              { 
                "use_open_max_il":false,
                "media_codec_direct":0,
                "player":"ANDROID_PLAYER",
                "use_mdeia_codec":true,
                "use_list_player":false
              },
              {
                "use_open_max_il":false,
                "media_codec_direct":0,
                "player":"VLC_PLAYER",
                "use_mdeia_codec":true,
                "use_list_player":false
              }
            ],
            "need_membuf":false,
            "is_downloaded":false,
            "description":"高清",
            "segment_list": [
              {
                "duration":0,
                "url":"",
                "bytes":0
              }
            ],
            "need_ringbuf":true,
            "user_agent":"Mozilla\/5.0 (Windows NT 6.1; WOW64) AppleWebKit\/537.36 (KHTML, like Gecko) Chrome\/34.0.1847.116 Safari\/537.36",
            "psedo_bitrate":0,
            "is_stub":false
          },

      data = new Blob([JSON.stringify(json)], {type: 'text/plain'});
      return $cordovaFile.writeFile(filePath, data, {append: false});
  }

  var getEntryJsonData = function(info, avid, title, quality) {

    var json = {
      "downloaded_bytes":0,
      "spid":0,
      "guessed_total_bytes":0,
      "avid":avid,
      "type_tag":"lua.flv.bapi.2",
      "total_bytes":0,
      "prefered_video_quality":quality || 200,
      "total_time_milli":0,
      "page_data": info,
      "seasion_id":0,
      "title":title,
      "is_completed":false,
      "is_primary_external_storage":true,
      "time_stamp":0
    }

    if (json.page_data.weblink) {
      json.page_data.weblink = null;
    };

    return new Blob([JSON.stringify(json)], {type: 'text/plain'});
  }

  var writeEntry = function(filePath, data) {
    return $cordovaFile.writeFile(filePath, data, {append: false});
  }

  var readEntry = function(filePath) {
    return $cordovaFile.readAsText(filePath)
            .then(function(text) {
              var json = JSON.parse(text);
              // json.storage_path .type_tag .title .avid
              //     .is_completed  .downloaded_bytes .total_bytes
              //     .page_data.page
              // 待定 fileDirUri == json.storage_path + json.type_tag
              return json;
            }, function() {
              // if read entry.json fail 
              // return an empty object to ensure $q.all() is resolved
              return {}
            })
  }

  var listBiliDir = function() {
    
    var uri = biliPath + "download";

    return $cordovaFile.listDir(uri)
    .then(function(entries) {
      // entries: avid dir like [{name: '10086'}, {name: '10087'}]
      var promises = entries.map(function(entry) {
        //every entry return a promise 
        //  - avInfo {Object} like {0: {}, 1: {}}
        return $cordovaFile.listDir(formatURI(entry.toURL()))
                .then(function(entries) {
                  // entries: page dir like [{name: '1'}, {name: '2'}]
                  var promises = entries.map(function(entry) {
                    //read every page's entry.json 
                    //return json {Object}
                    var entryJsonUri = formatURI(entry.toURL()) + "/entry.json";

                    return readEntry(entryJsonUri)
                  });

                  return $q.all(promises);
                })
                .then(function(infos) {
                  //filter the empty obj
                  var infos = infos.filter(function(info) {
                      return info.page_data;
                  });

                  infos.sort(function(a, b) {
                    return a.page_data.page > b.page_data.page ? 1 : -1;
                  })

                  var avInfo = {
                    avid: infos[0].avid,
                    title: infos[0].title,
                    pages: infos
                  };

                  return avInfo;
                }, function() {
                  // if read entry.json fail 
                  // return an empty object to ensure $q.all() is resolved
                  return {};
                })
      });

      return $q.all(promises);
    })
    .then(function(avInfos) {
      return avInfos;
    });
  }

  var avInfos = null; 
  return {
    list: function() {
      if (!avInfos) {
        avInfos = listBiliDir();
      }
      return avInfos;
    },
    refresh: function() {
      avInfos = listBiliDir();
      return avInfos;
    },
    add: function(avid, info, i, q) {

      var title = info.title,
          info = info.list[i];

      return addAVDir(avid)
              .then(function(entry) {
                return addAVPageDir(formatURI(entry.toURL()), info.page)
              })
              .then(function(entry) {
                
                var dirUri = formatURI(entry.toURL()),
                    entryJsonUri = dirUri + "entry.json";

                return $cordovaFile.checkFile(entryJsonUri)
                        .then(function() {

                        }, function(err) {
                          if (err.code == 1) {
                            return $cordovaFile.createFile(entryJsonUri)
                                    .then(function() {
                                      var data = getEntryJsonData(info, avid, title, q);
                                      return writeEntry(entryJsonUri, data)
                                    })
                          }
                        })
                        .then(function() {
                          return dirUri
                        })
              })
    },
    import: function(filePath, dirUri) {

      //dependence null
      var removeEntry = function(entry) {
        var q = $q.defer();
        entry.remove(function() {
          q.resolve();
        }, function() {
          q.reject();
        });
        return q.promise
      }
      //dependence null
      var removeDir = function(uri) {
        var q = $q.defer();
        $cordovaFile.checkDir(uri)
          .then(function(dirEntry) {
            dirEntry.removeRecursively(function() {
              q.resolve();
            }, function() {
              q.reject();
            });
          })
        return q.promise;
      }

      //dependence null
      var moveFile = function(fileEntry, dirEntry, newName) {

        var q = $q.defer();

        fileEntry.moveTo(dirEntry, newName, function() {
          q.resolve();
        }, function() {
          q.reject();
        })
        return q.promise;
      }

      //dependence : dirUri
      var importMP4 = function(fileEntry, typeTag) {

        var AVPageDirEntryPromise = $cordovaFile.checkDir(dirUri, false);

        AVPageDirEntryPromise.then(function(AVPageDirEntry) {
          //移动 mp4 视频文件
          var newName = typeTag + "_remux.mp4";
          return moveFile(fileEntry, AVPageDirEntry, newName)
        })
        .then(function() {
          //删除 typeTag 文件夹 eg: lua.flv.bapi.2
          var fileDirUri = dirUri + typeTag;
          return removeDir(fileDirUri)
        })
      }

      //dependence : dirUri
      var importFLV = function(fileEntry, typeTag) {

        var fileDirUri = dirUri + typeTag,
            fileDirEntryPromise = $cordovaFile.createDir(fileDirUri, false),
            fileDirListEntriesPromise = fileDirEntryPromise.then(function() {
                                            return $cordovaFile.listDir(fileDirUri)
                                          });

        var removeAll = function(entries) {
          var promises = entries.map(function(entry) {
            return removeEntry(entry)
          })
          return $q.all(promises);
        }


        //dependence : fileDirUri
        var createIndexJson = function() {
          var indexJsonUri = fileDirUri + "/index.json";

          return $cordovaFile.createFile(indexJsonUri, true)
                  .then(function() {
                    return writeIndexJson(indexJsonUri);
                  })
        }

        return fileDirListEntriesPromise
                .then(removeAll)
                .then(function() {
                  return fileDirEntryPromise.then(function(fileDirEntry) {
                    return moveFile(fileEntry, fileDirEntry, "0.flv")
                  })
                })
                .then(createIndexJson)
      }

      //dependence : dirUri
      var updateEntryJson = function(entryJson, metaData) {

        entryJson.is_completed = true;
        entryJson.downloaded_bytes = entryJson.total_bytes = metaData.size;

        var data = new Blob([JSON.stringify(entryJson)], {type: 'text/plain'});
        return $cordovaFile.writeFile(dirUri + "entry.json", data, {append: false});
      }

      var filePromise = $cordovaFile.checkFile(filePath);
      var fileMetaPromise = $cordovaFile.readFileMetadata(filePath);
      var typePromise = readEntry(dirUri + "entry.json");

      return $q.all([filePromise, typePromise, fileMetaPromise])
              .then(function(results) {
                var fileEntry = results[0];
                var entryJson = results[1], typeTag = entryJson.type_tag;
                var metaData = results[2];

                var promise;
                //导入 mp4
                if (fileEntry.name.substr(-4).toLowerCase() == ".mp4") {
                  promise = importMP4(fileEntry, typeTag);
                }
                //导入 其它类型
                promise = importFLV(fileEntry, typeTag);
                return promise.then(function() {
                  if (!entryJson.is_completed || !entryJson.total_bytes) {
                    return updateEntryJson(entryJson, metaData);
                  };
                  return
                })
              })
    }
  }
}])

.factory('Biliapi', ['$http', function($http) {

  var APPSECRET = '09e6cafa5f6999a801c7aa1a5dc64fe8',
      APPKEY = '23354a06e58b416c';

  var _getSign = function(params) {
    var s_keys = [];
    for (var param in params) {
      s_keys.push(param);
    }
    s_keys.sort();
    var data = "";
    for (var i = 0; i < s_keys.length; i++) {
      data+=(data ? "&" : "")+s_keys[i]+"="+encodeURIComponent(params[s_keys[i]]);
    }
    //hex_md5 source "js/md5.js"
    return hex_md5(data + APPSECRET);
  }


  return {
    getInfo: function(avid) {
      var params = {
        appkey: APPKEY,
        batch: 1,
        id: avid,
        page: 1
      };

      params.sign = _getSign(params);

      return $http.get('http://api.bilibili.com/view', {"params": params})
    },
    getUrl: function(cid, quality, format) {

      var quality = quality ? quality : 2,
          format = format ? format : "flv";

      var params = {
        appkey: APPKEY,
        cid: cid,
        platform: "bilime",
        otype: "json",
        quality: quality,
        type: format
      };

      return $http.get('http://interface.bilibili.com/playurl', {"params": params})
    }
  }
}])

/*
.factory('DownloadManager', ['Biliapi', function(Biliapi) {
  var downloadList = [];
}])
*/